//
//  追加プラグイン
//
//

/*:
 * @plugindesc ver1.01/スキルやアイテムの発動前に、スキルやアイテムに設定されたコモンイベントを発生させます。
 * @author Yana
 *
 * @param IndexVariableID
 * @desc 発動者のインデックスを保存する変数IDです。
 * 発動者がエネミーの場合は値に+1000されます。
 * @default 11
 *
 * @param TargetIndexVariableID
 * @desc 対象のインデックスを保存する変数IDです。
 * 対象が2体以上の場合は変数の値は-1が設定されます。
 * @default 12
 */

// コモンイベント発動用のなにもしないスキルは、光る演出をなくす
Game_Enemy.prototype.performActionStart = function (action) {
  if (typeof action.item().meta.NoneSkill === "undefined") {
    Game_Battler.prototype.performActionStart.call(this, action);
    this.requestEffect("whiten");
  }
};

// コモンイベント発動用のなにもしないスキルは、メッセージログのウェイトをなくす
Window_BattleLog.prototype.displayAction = function (subject, item) {
  var numMethods = this._methods.length;
  if (DataManager.isSkill(item)) {
    if (item.message1) {
      this.push("addText", subject.name() + item.message1.format(item.name));
    }
    if (item.message2) {
      this.push("addText", item.message2.format(item.name));
    }
  } else {
    this.push("addText", TextManager.useItem.format(subject.name(), item.name));
  }
  if (this._methods.length === numMethods && !item.meta.NoneSkill) {
    this.push("wait");
  }
};

// 空スキル呼び出し時のエラー落ち回避のため
BattleManager.processForcedAction = function () {
  if (this._actionForcedBattler) {
    this._turnForced = true;
    this._subject = this._actionForcedBattler;
    this._actionForcedBattler = null;

    if (!$gameParty.isAllDead()) {
      this.startAction();
    }
    this._subject.removeCurrentAction();
  }
};
