//=============================================================================
// NYA_ItemStock.js
//=============================================================================

/*:
 * @plugindesc アイテムリストを変数に格納します。
 * @help プラグインコマンドはありません。
 *
 * @param ItemId
 * @desc アイテムリストを指定した変数に格納します。
 * @default 1
 * @type number
 *
 * @param WeaponId
 * @desc 武器リストを指定した変数に格納します。
 * @default 2
 * @type number
 *
 * @param ArmorId
 * @desc 防具リストを指定した変数に格納します。
 * @default 3
 * @type number
 */

(function () {
  "use strict";

  //プラグインマネージャーで設定されたパラメータを取得
  var parameters = PluginManager.parameters("NYA_ItemStock");

  //パラメータを変数へ
  var itemId = Number(parameters["ItemId"]);
  var weaponId = Number(parameters["WeaponId"]);
  var armorId = Number(parameters["ArmorId"]);

  function convertEscape(txt) {
    return Window_Base.prototype.convertEscapeCharacters(txt);
  }

  var _Game_Interpreter_pluginCommand =
    Game_Interpreter.prototype.pluginCommand;
  Game_Interpreter.prototype.pluginCommand = function (command, args) {
    _Game_Interpreter_pluginCommand.call(this, command, args);

    if (command.toUpperCase() === "NYA_ITEMSTOCK") {
      if (args[0].toUpperCase() === "KEEP") {
        if (args[1]) {
          if (args[1].toUpperCase() === "ITEM") {
            // アイテムリストを変数#0001に保存
            $gameVariables.setValue(
              itemId,
              JsonEx.makeDeepCopy($gameParty._items)
            );
          }
          if (args[1].toUpperCase() === "WEAPON") {
            // 武器リストを変数#0002に保存
            $gameVariables.setValue(
              weaponId,
              JsonEx.makeDeepCopy($gameParty._weapons)
            );
          }
          if (args[1].toUpperCase() === "ARMOR") {
            // 防具リストを変数#0003に保存
            $gameVariables.setValue(
              armorId,
              JsonEx.makeDeepCopy($gameParty._armors)
            );
          }
          if (args[1].toUpperCase() === "ALL") {
            $gameVariables.setValue(
              itemId,
              JsonEx.makeDeepCopy($gameParty._items)
            );
            $gameVariables.setValue(
              weaponId,
              JsonEx.makeDeepCopy($gameParty._weapons)
            );
            $gameVariables.setValue(
              armorId,
              JsonEx.makeDeepCopy($gameParty._armors)
            );
          }
        } else {
          $gameVariables.setValue(
            itemId,
            JsonEx.makeDeepCopy($gameParty._items)
          );
          $gameVariables.setValue(
            weaponId,
            JsonEx.makeDeepCopy($gameParty._weapons)
          );
          $gameVariables.setValue(
            armorId,
            JsonEx.makeDeepCopy($gameParty._armors)
          );

          //                    console.log('$gameParty._items:'+$gameParty._items);

          //                    var list = [];
          //                    for (var id in $gameParty._items) {
          //                        console.log('$dataItems[id]:'+$dataItems[id].meta.nokeep);
          //                        if(!$dataItems[id].meta.nokeep){
          //                            list.push($dataItems[id]);
          //                        }
          //                    }
          //                    $gameVariables.setValue(itemId, JsonEx.makeDeepCopy(list));
        }
      }
      if (args[0].toUpperCase() === "CLEAR") {
        if (args[1]) {
          if (args[1].toUpperCase() === "ITEM") {
            // アイテムリストをクリア
            $gameParty._items = {};
          }
          if (args[1].toUpperCase() === "WEAPON") {
            // 武器リストをクリア
            $gameParty._weapons = {};
          }
          if (args[1].toUpperCase() === "ARMOR") {
            // 防具リストをクリア
            $gameParty._armors = {};
          }
          if (args[1].toUpperCase() === "ALL") {
            // アイテム・武器・防具をまとめてクリア
            $gameParty.initAllItems();
          }
        } else {
          // アイテム・武器・防具をまとめてクリア
          $gameParty.initAllItems();

          // アイテムリストをクリア
          //$gameParty._items = {};
          //アイテムからメモ欄に<nokeep>タグのあるものを除外してクリア
          //                    var list = [];
          //                    for (var id in $gameParty._items) {
          //                        if($dataItems[id].meta.nokeep){
          //                            console.log('アイテム:'+$dataItems[id].name+'をリストに戻した');
          //                            list.push($dataItems[id]);
          //                            //list.gainItem($dataItems[id], items[id]);
          //                        }
          //                    }
          //                    $gameParty._items = {};
          //                    $gameParty._items = list;

          //                    var items = $gameVariables.value(itemId);
          //                    Object.keys(items).forEach(function(id) {
          //                        console.log('$dataItems[id]:'+$dataItems[id].name+'    items[id]:'+items[id]);
          ////                        if(items[id].meta.nokeep){
          ////                            $gameParty.gainItem($dataItems[id], items[id]);
          ////                        }
          //                    });
        }
      }
      if (args[0].toUpperCase() === "RESTORE") {
        if (args[1]) {
          if (args[1].toUpperCase() === "ITEM") {
            // 変数#0001に保存したアイテムリストを復元
            $gameParty._items = JsonEx.makeDeepCopy(
              $gameVariables.value(itemId)
            );
          }
          if (args[1].toUpperCase() === "WEAPON") {
            // 変数#0002に保存した武器リストを復元
            $gameParty._weapons = JsonEx.makeDeepCopy(
              $gameVariables.value(weaponId)
            );
          }
          if (args[1].toUpperCase() === "ARMOR") {
            // 変数#0003に保存した防具リストを復元
            $gameParty._armors = JsonEx.makeDeepCopy(
              $gameVariables.value(armorId)
            );
          }
          if (args[1].toUpperCase() === "ALL") {
            // 変数#0001に保存したアイテムリストを復元
            $gameParty._items = JsonEx.makeDeepCopy(
              $gameVariables.value(itemId)
            );
            // 変数#0002に保存した武器リストを復元
            $gameParty._weapons = JsonEx.makeDeepCopy(
              $gameVariables.value(weaponId)
            );
            // 変数#0003に保存した防具リストを復元
            $gameParty._armors = JsonEx.makeDeepCopy(
              $gameVariables.value(armorId)
            );
          }
        } else {
          // 変数#0001に保存したアイテムリストを復元
          $gameParty._items = JsonEx.makeDeepCopy($gameVariables.value(itemId));
          // 変数#0002に保存した武器リストを復元
          $gameParty._weapons = JsonEx.makeDeepCopy(
            $gameVariables.value(weaponId)
          );
          // 変数#0003に保存した防具リストを復元
          $gameParty._armors = JsonEx.makeDeepCopy(
            $gameVariables.value(armorId)
          );
        }
      }
      if (args[0].toUpperCase() === "COMPOSITION") {
        if (args[1]) {
          if (
            args[1].toUpperCase() === "ITEM" ||
            args[1].toUpperCase() === "ALL"
          ) {
            // 変数#0001に保存したアイテムリストを、現在のアイテムリストに合成
            var items = $gameVariables.value(itemId);
            Object.keys(items).forEach(function (id) {
              $gameParty.gainItem($dataItems[id], items[id]);
            });
          }
          if (
            args[1].toUpperCase() === "WEAPON" ||
            args[1].toUpperCase() === "ALL"
          ) {
            // 変数#0002に保存した武器リストを、現在の武器リストに合成
            var weapons = $gameVariables.value(weaponId);
            Object.keys(weapons).forEach(function (id) {
              $gameParty.gainItem($dataWeapons[id], weapons[id]);
            });
          }
          if (
            args[1].toUpperCase() === "ARMOR" ||
            args[1].toUpperCase() === "ALL"
          ) {
            // 変数#0003に保存した防具リストを、現在の防具リストに合成
            var armors = $gameVariables.value(armorId);
            Object.keys(armors).forEach(function (id) {
              $gameParty.gainItem($dataArmors[id], armors[id]);
            });
          }
        } else {
          // 変数#0001に保存したアイテムリストを、現在のアイテムリストに合成
          var items = $gameVariables.value(itemId);
          Object.keys(items).forEach(function (id) {
            //                        console.log('$dataItems[id]:'+$dataItems[id].name+'    items[id]:'+items[id]);
            $gameParty.gainItem($dataItems[id], items[id]);
          });
          // 変数#0002に保存した武器リストを、現在の武器リストに合成
          var weapons = $gameVariables.value(weaponId);
          Object.keys(weapons).forEach(function (id) {
            $gameParty.gainItem($dataWeapons[id], weapons[id]);
          });
          // 変数#0003に保存した防具リストを、現在の防具リストに合成
          var armors = $gameVariables.value(armorId);
          Object.keys(armors).forEach(function (id) {
            $gameParty.gainItem($dataArmors[id], armors[id]);
          });
        }
      }
    }
  };
})();
