﻿//=============================================================================
// Yanfly Engine Plugins - Base Troop Events
// YEP_BaseTroopEvents.js
//=============================================================================

var Imported = Imported || {};
Imported.YEP_BaseTroopEvents = true;

var Yanfly = Yanfly || {};
Yanfly.BTE = Yanfly.BTE || {};
Yanfly.BTE.version = 1.01;

//=============================================================================
/*:ja
 * @plugindesc v1.01 全ての敵グループに対して、毎回戦闘で発生するイベントを設定します。
 * @author Yanfly Engine Plugins
 *
 * @param Base Troop ID
 * @text 基本敵グループID
 * @type troop
 * @desc 全戦闘の参照元となる敵グループのID
 * @default 1
 *
 * @help
 * 翻訳:ムノクラ
 * https://fungamemake.com/
 * https://twitter.com/munokura/
 *
 * ===========================================================================
 * 導入
 * ===========================================================================
 *
 * イベントページから、戦闘をカスタマイズしたい開発者向けのプラグインです。
 * 基本となる敵グループ内のイベントが、全ての戦闘で実行されます。
 * 一括で反映できるので、開発時間を短縮することが出来ます。
 *
 * ===========================================================================
 * Changelog
 * ===========================================================================
 *
 * Version 1.01:
 * - Updated for RPG Maker MV version 1.5.0.
 *
 * Version 1.00:
 * - Finished Plugin!
 */

//=============================================================================

//=============================================================================
// Parameter Variables
//=============================================================================

Yanfly.Parameters = PluginManager.parameters("YEP_BaseTroopEvents");
Yanfly.Param = Yanfly.Param || {};

Yanfly.Param.BaseTroopID = Number(Yanfly.Parameters["Base Troop ID"]);

//=============================================================================
// DataManager
//=============================================================================

Yanfly.BTE.DataManager_isDatabaseLoaded = DataManager.isDatabaseLoaded;
DataManager.isDatabaseLoaded = function () {
  if (!Yanfly.BTE.DataManager_isDatabaseLoaded.call(this)) return false;
  this.processBTEPages();
  return true;
};

DataManager.processBTEPages = function () {
  for (var n = 1; n < $dataTroops.length; n++) {
    var base_troop = $dataTroops[Yanfly.Param.BaseTroopID];
    var troop = $dataTroops[n];
    if (n !== Yanfly.Param.BaseTroopID && Yanfly.Param.BaseTroopID > 0) {
      if (troop._baseTroopEventsMade) continue;
      Yanfly.Util.extend(troop.pages, base_troop.pages);
      troop._baseTroopEventsMade = true;
    }
  }
};

//=============================================================================
// New Function
//=============================================================================

Yanfly.Util = Yanfly.Util || {};

Yanfly.Util.extend = function (mainArray, otherArray) {
  otherArray.forEach(function (i) {
    mainArray.push(i);
  }, this);
};

//=============================================================================
// End of File
//=============================================================================
